@@ -31,12 +31,12 @@
 #endif /* HAVE_STRING_H */
 
 #include <freetds/utils.h>
+#include <freetds/replacements.h>
 
 #include "bkpublic.h"
 
 #include "ctpublic.h"
 #include "ctlib.h"
-#include "replacements.h"
 
 static void _blk_null_error(TDSBCPINFO *bcpinfo, int index, int offset);
 static TDSRET _blk_get_col_data(TDSBCPINFO *bulk, TDSCOLUMN *bcpcol, int offset);
@@ -55,6 +55,12 @@ blk_alloc(CS_CONNECTION * connection, CS_INT version, CS_BLKDESC ** blk_pointer)
 
 	tdsdump_log(TDS_DBG_FUNC, "blk_alloc(%p, %d, %p)\n", connection, version, blk_pointer);
 
+	if (!connection || !connection->tds_socket)
+		return CS_FAIL;
+
+	if (connection->tds_socket->conn->tds_version < 0x500)
+		return CS_FAIL;
+
 	blkdesc = (CS_BLKDESC *) tds_alloc_bcpinfo();
 	if (!blkdesc)
 		return CS_FAIL;
@@ -608,7 +614,7 @@ static TDSRET
 _blk_get_col_data(TDSBCPINFO *bulk, TDSCOLUMN *bindcol, int offset)
 {
 	int result = 0;
-	CS_INT null_column = 0;
+	bool null_column = false;
 	unsigned char *src = NULL;
 
 	CS_INT      srctype = 0;
@@ -680,7 +686,7 @@ _blk_get_col_data(TDSBCPINFO *bulk, TDSCOLUMN *bindcol, int offset)
 	}
 	if (srclen == 0) {
 		if (nullind && *nullind == -1)
-			null_column = 1;
+			null_column = true;
 	}
 
 	if (!null_column) {
