@@ -44,7 +44,7 @@
 #include <freetds/tds.h>
 #include <freetds/convert.h>
 #include <freetds/utils/string.h>
-#include <replacements.h>
+#include <freetds/replacements.h>
 #include <sybfront.h>
 #include <sybdb.h>
 #include <dblib.h>
