@@ -1,4 +1,4 @@
-create table #dblib0016 (f1 int not null, str nvarchar(5), uuid uniqueidentifier)
+create table #dblib0016 (b1 bit, b2 bit, i1 int not null, b3 bit, b4 bit)
 go
 select * from #dblib0016 where 0=1
 go
@@ -6,4 +6,3 @@ select * from #dblib0016 where 0=1
 go
 drop table #dblib0016
 go
-
