@@ -30,7 +30,7 @@
 #include <freetds/odbc.h>
 #include <freetds/utils/string.h>
 #include <freetds/utils.h>
-#include "replacements.h"
+#include <freetds/replacements.h>
 
 #define ODBC_PARAM(p) static const char odbc_param_##p[] = #p;
 ODBC_PARAM_LIST
@@ -251,6 +251,9 @@ odbc_get_dsn_info(TDS_ERRS *errs, const char *DSN, TDSLOGIN * login)
 	if (myGetPrivateProfileString(DSN, odbc_param_AttachDbFilename, tmp) > 0)
 		tds_parse_conf_section(TDS_STR_DBFILENAME, tmp, login);
 
+	if (myGetPrivateProfileString(DSN, odbc_param_Timeout, tmp) > 0)
+		tds_parse_conf_section(TDS_STR_TIMEOUT, tmp, login);
+
 	return 1;
 }
 
@@ -441,6 +444,8 @@ odbc_parse_connect_string(TDS_ERRS *errs, const char *connect_string, const char
 
 			tds_parse_conf_section(TDS_STR_READONLY_INTENT, readonly_intent, login);
 			tdsdump_log(TDS_DBG_INFO1, "Application Intent %s\n", readonly_intent);
+		} else if (CHK_PARAM(Timeout)) {
+			tds_parse_conf_section(TDS_STR_TIMEOUT, tds_dstr_cstr(&value), login);
 		}
 
 		if (num_param >= 0 && parsed_params) {
@@ -834,11 +839,14 @@ ODBCINSTGetProperties(HODBCINSTPROPERTY hLastProperty)
 		"Appends dump file instead of overwriting it. Useful for debugging when many processes are active.");
 
 	hLastProperty = definePropertyString(hLastProperty, odbc_param_DebugFlags, "", 
-		"Sets granularity of logging. A set of bit that specify levels and information. See table below for bit specification.");
+		"Sets granularity of logging. A set of bit that specify levels and informations. See table below for bit specification.");
 
 	hLastProperty = definePropertyList(hLastProperty, odbc_param_Encryption, TDS_STR_ENCRYPTION_OFF, aEncryption, sizeof(aEncryption),
 		"The encryption method.");
 
+	hLastProperty = definePropertyString(hLastProperty, odbc_param_Timeout, "10",
+		"Connection timeout.");
+
 	return 1;
 }
 
