@@ -1,5 +1,6 @@
 #include "common.h"
 
+#include <stdarg.h>
 #include <assert.h>
 #include <ctype.h>
 
@@ -25,7 +26,7 @@
 #define TDS_SDIR_SEPARATOR "\\"
 #endif
 
-#include "replacements.h"
+#include <freetds/replacements.h>
 
 HENV odbc_env;
 HDBC odbc_conn;
@@ -60,20 +61,6 @@ check_lib(char *path, const char *file)
 	return 0;
 }
 
-/* some platforms do not have setenv, define a replacement */
-#if !HAVE_SETENV
-void
-odbc_setenv(const char *name, const char *value, int overwrite)
-{
-#if HAVE_PUTENV
-	char buf[1024];
-
-	sprintf(buf, "%s=%s", name, value);
-	putenv(buf);
-#endif
-}
-#endif
-
 /* this should be extended with all possible systems... */
 static const char *const search_driver[] = {
 	".libs/libtdsodbc.so",
@@ -702,17 +689,23 @@ odbc_from_sqlwchar(char *dst, const SQLWCHAR *src, int n)
 ODBC_BUF *odbc_buf = NULL;
 
 void *
-odbc_buf_get(ODBC_BUF** buf, size_t s)
+odbc_buf_add(ODBC_BUF** buf, void *ptr)
 {
 	ODBC_BUF *p = (ODBC_BUF*) calloc(1, sizeof(ODBC_BUF));
+	assert(ptr);
 	assert(p);
-	p->buf = malloc(s);
-	assert(p->buf);
+	p->buf = ptr;
 	p->next = *buf;
 	*buf = p;
 	return p->buf;
 }
 
+void *
+odbc_buf_get(ODBC_BUF** buf, size_t s)
+{
+	return odbc_buf_add(buf, malloc(s));
+}
+
 void
 odbc_buf_free(ODBC_BUF** buf)
 {
@@ -753,6 +746,19 @@ odbc_get_sqlchar(ODBC_BUF** buf, SQLWCHAR *s)
 	return out;
 }
 
+char *
+odbc_buf_asprintf(ODBC_BUF** buf, const char *fmt, ...)
+{
+	va_list ap;
+	char *ret = NULL;
+
+	va_start(ap, fmt);
+	vasprintf(&ret, fmt, ap);
+	va_end(ap);
+
+	return (char *) odbc_buf_add(buf, ret);
+}
+
 typedef union {
 	struct sockaddr sa;
 	struct sockaddr_in sin;
