@@ -27,6 +27,7 @@
 
 #include <freetds/sysdep_private.h>
 #include <freetds/macros.h>
+#include <freetds/replacements.h>
 
 #ifndef HAVE_SQLLEN
 #ifndef SQLULEN
@@ -177,12 +178,6 @@ unsigned int odbc_db_version_int(void);
 int odbc_driver_is_freetds(void);
 int odbc_tds_version(void);
 
-#if !HAVE_SETENV
-void odbc_setenv(const char *name, const char *value, int overwrite);
-
-#define setenv odbc_setenv
-#endif
-
 void odbc_mark_sockets_opened(void);
 TDS_SYS_SOCKET odbc_find_last_socket(void);
 
@@ -200,6 +195,7 @@ typedef struct odbc_buf{
 	void *buf;
 } ODBC_BUF;
 extern ODBC_BUF *odbc_buf;
+void *odbc_buf_add(ODBC_BUF** buf, void *ptr);
 void *odbc_buf_get(ODBC_BUF** buf, size_t s);
 void odbc_buf_free(ODBC_BUF** buf);
 #define ODBC_GET(s)  odbc_buf_get(&odbc_buf, s)
@@ -219,6 +215,8 @@ char *odbc_get_sqlchar(ODBC_BUF** buf, SQLWCHAR *s);
 #define C(s) ((char*)(s))
 #endif
 
+char *odbc_buf_asprintf(ODBC_BUF** buf, const char *fmt, ...);
+
 struct odbc_lookup_int
 {
 	const char *name;
