@@ -30,6 +30,19 @@ static char prepare_before = 0;
 static char use_cursors = 0;
 static int only_test = 0;
 
+static const char *
+split_collate(ODBC_BUF** buf, const char **type)
+{
+	const char *collate = "", *p;
+
+	if ((p = strstr(*type, " COLLATE ")) != NULL) {
+		if (odbc_db_is_microsoft())
+			collate = p;
+		*type = odbc_buf_asprintf(buf, "%.*s", (int) (p - *type), *type);
+	}
+	return collate;
+}
+
 static int
 TestOutput(const char *type, const char *value_to_convert, SQLSMALLINT out_c_type, SQLSMALLINT out_sql_type, const char *expected)
 {
@@ -37,15 +50,18 @@ TestOutput(const char *type, const char *value_to_convert, SQLSMALLINT out_c_typ
 	unsigned char out_buf[256];
 	SQLLEN out_len = 0;
 	const char *sep;
+	const char *collate;
 
 	odbc_reset_statement();
 
 	/* build store procedure to test */
 	odbc_command("IF OBJECT_ID('spTestProc') IS NOT NULL DROP PROC spTestProc");
 	sep = "'";
-	if (strncmp(value_to_convert, "0x", 2) == 0)
+	if (strncmp(value_to_convert, "0x", 2) == 0 || strncmp(value_to_convert, "NCHAR(", 6) == 0)
 		sep = "";
-	sprintf(sbuf, "CREATE PROC spTestProc @i %s OUTPUT AS SELECT @i = CONVERT(%s, %s%s%s)", type, type, sep, value_to_convert, sep);
+	collate = split_collate(&odbc_buf, &type);
+	sprintf(sbuf, "CREATE PROC spTestProc @i %s OUTPUT AS SELECT @i = CONVERT(%s, %s%s%s)%s",
+		type, type, sep, value_to_convert, sep, collate);
 	odbc_command(sbuf);
 	memset(out_buf, 0, sizeof(out_buf));
 
@@ -103,6 +119,15 @@ TestOutput(const char *type, const char *value_to_convert, SQLSMALLINT out_c_typ
 static char check_truncation = 0;
 static char use_nts = 0;
 
+/**
+ * Test a parameter as input to prepared statement
+ *
+ * "value_to_convert" will be inserted as database type "type" and C type "out_c_type" into a column of
+ * database type "param_type" and SQL type "out_sql_type". Then a select statement check that the column
+ * in the database is the same as "value_to_convert".
+ * "value_to_convert" can also contain a syntax like "input -> output", in this case the initial inserted
+ * value is "input" and the final check will check for "output".
+ */
 static void
 TestInput(SQLSMALLINT out_c_type, const char *type, SQLSMALLINT out_sql_type, const char *param_type, const char *value_to_convert)
 {
@@ -113,6 +138,7 @@ TestInput(SQLSMALLINT out_c_type, const char *type, SQLSMALLINT out_sql_type, co
 	size_t value_len = strlen(value_to_convert);
 	const char *p;
 	const char *sep = "'";
+	const char *collate;
 
 	odbc_reset_statement();
 
@@ -123,7 +149,8 @@ TestInput(SQLSMALLINT out_c_type, const char *type, SQLSMALLINT out_sql_type, co
 	}
 	if (value_len >= 2 && strncmp(value_to_convert, "0x", 2) == 0)
 		sep = "";
-	sprintf(sbuf, "SELECT CONVERT(%s, %s%.*s%s)", type, sep, (int) value_len, value_to_convert, sep);
+	collate = split_collate(&odbc_buf, &type);
+	sprintf(sbuf, "SELECT CONVERT(%s, %s%.*s%s%s)", type, sep, (int) value_len, value_to_convert, sep, collate);
 	odbc_command(sbuf);
 	SQLBindCol(odbc_stmt, 1, out_c_type, out_buf, sizeof(out_buf), &out_len);
 	CHKFetch("SI");
@@ -136,7 +163,8 @@ TestInput(SQLSMALLINT out_c_type, const char *type, SQLSMALLINT out_sql_type, co
 
 	/* create a table with a column of that type */
 	odbc_reset_statement();
-	sprintf(sbuf, "CREATE TABLE #tmp_insert (col %s)", param_type);
+	collate = split_collate(&odbc_buf, &param_type);
+	sprintf(sbuf, "CREATE TABLE #tmp_insert (col %s%s)", param_type, collate);
 	odbc_command(sbuf);
 
 	if (use_cursors) {
@@ -171,18 +199,23 @@ TestInput(SQLSMALLINT out_c_type, const char *type, SQLSMALLINT out_sql_type, co
 
 	/* check if row is present */
 	if (!check_truncation) {
+		char *p;
+
 		odbc_reset_statement();
 		sep = "'";
 		if (strncmp(expected, "0x", 2) == 0)
 			sep = "";
+
+		strcpy(sbuf, "SELECT * FROM #tmp_insert WHERE ");
+		p = strchr(sbuf, 0);
 		if (strcmp(param_type, "TEXT") == 0)
-			sprintf(sbuf, "SELECT * FROM #tmp_insert WHERE CONVERT(VARCHAR(255), col) = CONVERT(VARCHAR(255), %s%s%s)", sep, expected, sep);
+			sprintf(p, "CONVERT(VARCHAR(255), col) = CONVERT(VARCHAR(255), %s%s%s)", sep, expected, sep);
 		else if (strcmp(param_type, "NTEXT") == 0)
-			sprintf(sbuf, "SELECT * FROM #tmp_insert WHERE CONVERT(NVARCHAR(2000), col) = CONVERT(NVARCHAR(2000), %s%s%s)", sep, expected, sep);
+			sprintf(p, "CONVERT(NVARCHAR(2000), col) = CONVERT(NVARCHAR(2000), %s%s%s)", sep, expected, sep);
 		else if (strcmp(param_type, "IMAGE") == 0)
-			sprintf(sbuf, "SELECT * FROM #tmp_insert WHERE CONVERT(VARBINARY(255), col) = CONVERT(VARBINARY(255), %s%s%s)", sep, expected, sep);
+			sprintf(p, "CONVERT(VARBINARY(255), col) = CONVERT(VARBINARY(255), %s%s%s)", sep, expected, sep);
 		else
-			sprintf(sbuf, "SELECT * FROM #tmp_insert WHERE col = CONVERT(%s, %s%s%s)", param_type, sep, expected, sep);
+			sprintf(p, "col = CONVERT(%s, %s%s%s)", param_type, sep, expected, sep);
 		odbc_command(sbuf);
 
 		CHKFetch("S");
@@ -384,7 +417,7 @@ AllTests(void)
 	TestInput(SQL_C_NUMERIC, "NUMERIC(20,3)", SQL_LONGVARCHAR, "TEXT", "578246.234 -> 578246");
 
 	TestInput(SQL_C_CHAR, "VARCHAR(100)", SQL_VARBINARY, "VARBINARY(20)", "4145544F -> AETO");
-	TestInput(SQL_C_CHAR, "TEXT", SQL_VARBINARY, "VARBINARY(20)", "4145544F -> AETO");
+	TestInput(SQL_C_CHAR, "TEXT COLLATE Latin1_General_CI_AS", SQL_VARBINARY, "VARBINARY(20)", "4145544F -> AETO");
 	TestInput(SQL_C_CHAR, "VARCHAR(100)", SQL_LONGVARBINARY, "IMAGE", "4145544F -> AETO");
 	TestInput(SQL_C_BINARY, "VARBINARY(100)", SQL_VARCHAR, "VARCHAR(20)", "0x4145544F -> AETO");
 	TestInput(SQL_C_BINARY, "IMAGE", SQL_VARCHAR, "VARCHAR(20)", "0x4145544F -> AETO");
@@ -409,7 +442,7 @@ AllTests(void)
 	/* while usually on Microsoft database this encoding is valid on Sybase the database
 	 * could use UTF-8 encoding where \xf8\xf9 is an invalid encoded string */
 	if (odbc_db_is_microsoft() && odbc_tds_version() > 0x700)
-		TestOutput("VARCHAR(20)", "0xf8f9", SQL_C_CHAR, SQL_VARCHAR, "2 \xf8\xf9");
+		TestOutput("VARCHAR(20) COLLATE Latin1_General_CI_AS", "NCHAR(0xf8) + NCHAR(0xf9)", SQL_C_CHAR, SQL_VARCHAR, "2 \xf8\xf9");
 
 	/* MSSQL 2000 using ptotocol 7.1+ */
 	if ((odbc_db_is_microsoft() && odbc_db_version_int() >= 0x08000000u && odbc_tds_version() > 0x700)
@@ -442,7 +475,7 @@ AllTests(void)
 		TestOutput("NVARCHAR(20)", "foo test", SQL_C_CHAR, SQL_WVARCHAR, "6 foo te");
 		precision = 12;
 		TestOutput("NVARCHAR(20)", "foo test", SQL_C_CHAR, SQL_WVARCHAR, "8 foo test");
-		/* TODO use collate in sintax if available */
+		/* TODO use collate in syntax if available */
 		TestOutput("NVARCHAR(20)", "0xf800f900", SQL_C_CHAR, SQL_WVARCHAR, "2 \xf8\xf9");
 
 		TestInput(SQL_C_WCHAR, "NVARCHAR(10)", SQL_WVARCHAR, "NVARCHAR(10)", "1EasyTest2");
@@ -457,7 +490,7 @@ AllTests(void)
 		TestInput(SQL_C_WCHAR, "NVARCHAR(10)", SQL_INTEGER, "INT", " -423785  -> -423785");
 
 		TestInput(SQL_C_CHAR, "NVARCHAR(100)", SQL_VARBINARY, "VARBINARY(20)", "4145544F -> AETO");
-		TestInput(SQL_C_CHAR, "NTEXT", SQL_VARBINARY, "VARBINARY(20)", "4145544F -> AETO");
+		TestInput(SQL_C_CHAR, "NTEXT COLLATE Latin1_General_CI_AS", SQL_VARBINARY, "VARBINARY(20)", "4145544F -> AETO");
 
 		TestInput(SQL_C_BINARY, "VARBINARY(100)", SQL_WVARCHAR, "NVARCHAR(20)", "0x4100450054004F00 -> AETO");
 		TestInput(SQL_C_BINARY, "IMAGE", SQL_WVARCHAR, "NVARCHAR(20)", "0x4100450054004F00 -> AETO");
