@@ -2,35 +2,11 @@
 
 /* Test for SQLPutData */
 
-static char software_version[] = "$Id: putdata.c,v 1.18 2010/07/05 09:20:33 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 static const char test_text[] =
 	"Nel mezzo del cammin di nostra vita\n" "mi ritrovai per una selva oscura\n" "che' la diritta via era smarrita.";
 
 #define BYTE_AT(n) (((n) * 245 + 123) & 0xff)
 
-static void
-CheckNoRow(const char *query)
-{
-	SQLRETURN rc;
-
-	rc = CHKExecDirect((SQLCHAR *) query, SQL_NTS, "SINo");
-	if (rc == SQL_NO_DATA)
-		return;
-
-	do {
-		SQLSMALLINT cols;
-
-		CHKNumResultCols(&cols, "S");
-		if (cols != 0) {
-			fprintf(stderr, "Data not expected here, query:\n\t%s\n", query);
-			odbc_disconnect();
-			exit(1);
-		}
-	} while (CHKMoreResults("SNo") == SQL_SUCCESS);
-}
-
 static int
 to_sqlwchar(SQLWCHAR *dst, const char *src, int n)
 {
@@ -42,8 +18,8 @@ to_sqlwchar(SQLWCHAR *dst, const char *src, int n)
 
 static char sql[1024];
 
-int
-main(int argc, char *argv[])
+static void
+Test(int direct)
 {
 	SQLLEN ind;
 	int len = strlen(test_text), n, i;
@@ -54,9 +30,8 @@ main(int argc, char *argv[])
 	SQLRETURN RetCode;
 	int type, lc, sql_type;
 
-	odbc_connect();
-
 	/* create table to hold data */
+	odbc_command_with_result(odbc_stmt, "DROP TABLE #putdata");
 	odbc_command("CREATE TABLE #putdata (c TEXT NULL, b IMAGE NULL)");
 
 	sql_type = SQL_LONGVARCHAR;
@@ -71,15 +46,22 @@ main(int argc, char *argv[])
 		 * test for char 
 		 */
 
-		CHKPrepare((SQLCHAR *) "INSERT INTO #putdata(c) VALUES(?)", SQL_NTS, "S");
+		if (!direct) {
+			CHKPrepare(T("INSERT INTO #putdata(c) VALUES(?)"), SQL_NTS, "S");
 
-		CHKExecute("Ne");
+			CHKExecute("Ne");
+		} else {
+			CHKExecDirect(T("INSERT INTO #putdata(c) VALUES(?)"), SQL_NTS, "Ne");
+		}
 
 		p = test_text;
 		n = 5;
+		ptr = ((char*)0) + 0xdeadbeef;
 		CHKParamData(&ptr, "Ne");
-		if (ptr != (SQLPOINTER) 123)
+		if (ptr != (SQLPOINTER) 123) {
+			fprintf(stderr, "%p\n", ptr);
 			ODBC_REPORT_ERROR("Wrong pointer from SQLParamData");
+		}
 		while (*p) {
 			int l = strlen(p);
 
@@ -125,7 +107,7 @@ main(int argc, char *argv[])
 	CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_BINARY, SQL_LONGVARBINARY, 0, 0, (SQLPOINTER) 4567, 0, &ind, "S");
 	ind = SQL_LEN_DATA_AT_EXEC(254);
 
-	CHKPrepare((SQLCHAR *) "UPDATE #putdata SET b = ?", SQL_NTS, "S");
+	CHKPrepare(T("UPDATE #putdata SET b = ?"), SQL_NTS, "S");
 
 	CHKExecute("Ne");
 
@@ -168,28 +150,33 @@ main(int argc, char *argv[])
 			*pp++ = *p;
 	} while(*p++);
 	strcat(sql, "') SELECT 1");
-	CheckNoRow(sql);
+	odbc_check_no_row(sql);
 
 	odbc_command("DELETE FROM #putdata");
 
 	/* test len == 0 case from ML */
 	type = SQL_C_CHAR;
 	for (;;) {
-		CHKPrepare((SQLCHAR *) "INSERT INTO #putdata(c) VALUES(?)", SQL_NTS, "S");
+		if (!direct)
+			CHKPrepare(T("INSERT INTO #putdata(c) VALUES(?)"), SQL_NTS, "S");
 
 		CHKBindParameter(1, SQL_PARAM_INPUT, type, SQL_LONGVARCHAR, 0, 0, (PTR) 2, 0, &ind, "S");
 
 		ind = SQL_LEN_DATA_AT_EXEC(0);
 
-		RetCode = CHKExecute("Ne");
+		if (!direct)
+			CHKExecute("Ne");
+		else
+			CHKExecDirect(T("INSERT INTO #putdata(c) VALUES(?)"), SQL_NTS, "Ne");
+		RetCode = SQL_NEED_DATA;
 		while (RetCode == SQL_NEED_DATA) {
 			RetCode = SQLParamData(odbc_stmt, &ptr);
 			if (RetCode == SQL_NEED_DATA) {
 				if (type == SQL_C_CHAR) {
-					SQLPutData(odbc_stmt, "abc", 3);
+					CHKPutData("abc", 3, "S");
 				} else {
 					SQLWCHAR buf[10];
-					SQLPutData(odbc_stmt, buf, to_sqlwchar(buf, "abc", 3));
+					CHKPutData(buf, to_sqlwchar(buf, "abc", 3), "S");
 				}
 			}
 		}
@@ -200,9 +187,51 @@ main(int argc, char *argv[])
 	}
 
 	/* check inserts ... */
-	CheckNoRow("IF EXISTS(SELECT * FROM #putdata WHERE c NOT LIKE 'abc') SELECT 1");
+	odbc_check_no_row("IF EXISTS(SELECT * FROM #putdata WHERE c NOT LIKE 'abc') SELECT 1");
+
+	odbc_command("DELETE FROM #putdata");
+
+	/* test putting 0 bytes from Sebastien Flaesch */
+	if (odbc_db_is_microsoft()) {
+		type = SQL_C_CHAR;
+		for (;;) {
+			if (!direct)
+				CHKPrepare(T("INSERT INTO #putdata(c) VALUES(?)"), SQL_NTS, "S");
+
+			CHKBindParameter(1, SQL_PARAM_INPUT, type, SQL_LONGVARCHAR, 10, 0, (PTR) 2, 10, &ind, "S");
+
+			ind = SQL_DATA_AT_EXEC;
+
+			if (!direct)
+				CHKExecute("Ne");
+			else
+				CHKExecDirect(T("INSERT INTO #putdata(c) VALUES(?)"), SQL_NTS, "Ne");
+			RetCode = SQL_NEED_DATA;
+			while (RetCode == SQL_NEED_DATA) {
+				RetCode = SQLParamData(odbc_stmt, &ptr);
+				if (RetCode == SQL_NEED_DATA)
+					CHKPutData("", 0, "S");
+			}
+			if (type != SQL_C_CHAR)
+				break;
+			type = SQL_C_WCHAR;
+			odbc_reset_statement();
+		}
+
+		/* check inserts ... */
+		odbc_check_no_row("IF EXISTS(SELECT * FROM #putdata WHERE c NOT LIKE '') SELECT 1");
+	}
 
 	/* TODO test cancel inside SQLExecute */
+}
+
+int
+main(int argc, char *argv[])
+{
+	odbc_connect();
+
+	Test(0);
+	Test(1);
 
 	odbc_disconnect();
 
