@@ -44,7 +44,7 @@
 #include <freetds/iconv.h>
 #include <freetds/stream.h>
 #include <freetds/utils/string.h>
-#include "replacements.h"
+#include <freetds/replacements.h>
 
 /** \cond HIDDEN_SYMBOLS */
 #ifndef MAX
@@ -67,8 +67,10 @@ typedef struct tds_pbcb
 
 static TDSRET tds7_bcp_send_colmetadata(TDSSOCKET *tds, TDSBCPINFO *bcpinfo);
 static TDSRET tds_bcp_start_insert_stmt(TDSSOCKET *tds, TDSBCPINFO *bcpinfo);
-static int tds_bcp_add_fixed_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data, tds_bcp_null_error null_error, int offset, unsigned char * rowbuffer, int start);
-static int tds_bcp_add_variable_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data, tds_bcp_null_error null_error, int offset, TDS_UCHAR *rowbuffer, int start, int *pncols);
+static int tds5_bcp_add_fixed_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data, tds_bcp_null_error null_error,
+				      int offset, unsigned char * rowbuffer, int start);
+static int tds5_bcp_add_variable_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data, tds_bcp_null_error null_error,
+					 int offset, TDS_UCHAR *rowbuffer, int start, int *pncols);
 static void tds_bcp_row_free(TDSRESULTINFO* result, unsigned char *row);
 
 /**
@@ -324,6 +326,135 @@ tds_bcp_start_insert_stmt(TDSSOCKET * tds, TDSBCPINFO * bcpinfo)
 	return TDS_SUCCESS;
 }
 
+static TDSRET
+tds7_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data, int offset)
+{
+	int i;
+
+	tds_put_byte(tds, TDS_ROW_TOKEN);   /* 0xd1 */
+	for (i = 0; i < bcpinfo->bindinfo->num_cols; i++) {
+
+		TDS_INT save_size;
+		unsigned char *save_data;
+		TDSBLOB blob;
+		TDSCOLUMN  *bindcol;
+		TDSRET rc;
+
+		bindcol = bcpinfo->bindinfo->columns[i];
+
+		/*
+		 * Don't send the (meta)data for timestamp columns or
+		 * identity columns unless indentity_insert is enabled.
+		 */
+
+		if ((!bcpinfo->identity_insert_on && bindcol->column_identity) ||
+			bindcol->column_timestamp ||
+			bindcol->column_computed) {
+			continue;
+		}
+
+		rc = get_col_data(bcpinfo, bindcol, offset);
+		if (TDS_FAILED(rc)) {
+			tdsdump_log(TDS_DBG_INFO1, "get_col_data (column %d) failed\n", i + 1);
+			return rc;
+		}
+		tdsdump_log(TDS_DBG_INFO1, "gotten column %d length %d null %d\n",
+				i + 1, bindcol->bcp_column_data->datalen, bindcol->bcp_column_data->is_null);
+
+		save_size = bindcol->column_cur_size;
+		save_data = bindcol->column_data;
+		assert(bindcol->column_data == NULL);
+		if (bindcol->bcp_column_data->is_null) {
+			bindcol->column_cur_size = -1;
+		} else if (is_blob_col(bindcol)) {
+			bindcol->column_cur_size = bindcol->bcp_column_data->datalen;
+			memset(&blob, 0, sizeof(blob));
+			blob.textvalue = (TDS_CHAR *) bindcol->bcp_column_data->data;
+			bindcol->column_data = (unsigned char *) &blob;
+		} else {
+			bindcol->column_cur_size = bindcol->bcp_column_data->datalen;
+			bindcol->column_data = bindcol->bcp_column_data->data;
+		}
+		rc = bindcol->funcs->put_data(tds, bindcol, 1);
+		bindcol->column_cur_size = save_size;
+		bindcol->column_data = save_data;
+
+		if (TDS_FAILED(rc))
+			return rc;
+	}
+	return TDS_SUCCESS;
+}
+
+static TDSRET
+tds5_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo,
+		 tds_bcp_get_col_data get_col_data, tds_bcp_null_error null_error, int offset)
+{
+	int row_pos;
+	int row_sz_pos;
+	int blob_cols = 0;
+	int var_cols_written = 0;
+	TDS_INT	 old_record_size = bcpinfo->bindinfo->row_size;
+	unsigned char *record = bcpinfo->bindinfo->current_row;
+	int i;
+
+	memset(record, '\0', old_record_size);	/* zero the rowbuffer */
+
+	/*
+	 * offset 0 = number of var columns
+	 * offset 1 = row number.  zeroed (datasever assigns)
+	 */
+	row_pos = 2;
+
+	if ((row_pos = tds5_bcp_add_fixed_columns(bcpinfo, get_col_data, null_error, offset, record, row_pos)) < 0)
+		return TDS_FAIL;
+
+	row_sz_pos = row_pos;
+
+	/* potential variable columns to write */
+
+	row_pos = tds5_bcp_add_variable_columns(bcpinfo, get_col_data, null_error, offset, record, row_pos, &var_cols_written);
+	if (row_pos < 0)
+		return TDS_FAIL;
+
+
+	if (var_cols_written) {
+		TDS_PUT_UA2LE(&record[row_sz_pos], row_pos);
+		record[0] = var_cols_written;
+	}
+
+	tdsdump_log(TDS_DBG_INFO1, "old_record_size = %d new size = %d \n", old_record_size, row_pos);
+
+	tds_put_smallint(tds, row_pos);
+	tds_put_n(tds, record, row_pos);
+
+	/* row is done, now handle any text/image data */
+
+	blob_cols = 0;
+
+	for (i = 0; i < bcpinfo->bindinfo->num_cols; i++) {
+		TDSCOLUMN  *bindcol = bcpinfo->bindinfo->columns[i];
+		if (is_blob_type(bindcol->column_type)) {
+			TDSRET rc = get_col_data(bcpinfo, bindcol, offset);
+			if (TDS_FAILED(rc))
+				return rc;
+			/* unknown but zero */
+			tds_put_smallint(tds, 0);
+			tds_put_byte(tds, bindcol->column_type);
+			tds_put_byte(tds, 0xff - blob_cols);
+			/*
+			 * offset of txptr we stashed during variable
+			 * column processing
+			 */
+			tds_put_smallint(tds, bindcol->column_textpos);
+			tds_put_int(tds, bindcol->bcp_column_data->datalen);
+			tds_put_n(tds, bindcol->bcp_column_data->data, bindcol->bcp_column_data->datalen);
+			blob_cols++;
+
+		}
+	}
+	return TDS_SUCCESS;
+}
+
 /**
  * Send one row of data to server
  * \tds
@@ -334,142 +465,34 @@ tds_bcp_start_insert_stmt(TDSSOCKET * tds, TDSBCPINFO * bcpinfo)
  * \return TDS_SUCCESS or TDS_FAIL.
  */
 TDSRET
-tds_bcp_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data, tds_bcp_null_error ignored, int offset)
+tds_bcp_send_record(TDSSOCKET *tds, TDSBCPINFO *bcpinfo,
+		    tds_bcp_get_col_data get_col_data, tds_bcp_null_error null_error, int offset)
 {
-	TDSCOLUMN  *bindcol;
-	int i;
 	TDSRET rc;
 
-	tdsdump_log(TDS_DBG_FUNC, "tds_bcp_send_bcp_record(%p, %p, %p, ignored, %d)\n", tds, bcpinfo, get_col_data, offset);
+	tdsdump_log(TDS_DBG_FUNC, "tds_bcp_send_bcp_record(%p, %p, %p, %p, %d)\n",
+		    tds, bcpinfo, get_col_data, null_error, offset);
 
 	if (tds->out_flag != TDS_BULK || tds_set_state(tds, TDS_WRITING) != TDS_WRITING)
 		return TDS_FAIL;
 
-	if (IS_TDS7_PLUS(tds->conn)) {
+	if (IS_TDS7_PLUS(tds->conn))
+		rc = tds7_send_record(tds, bcpinfo, get_col_data, offset);
+	else
+		rc = tds5_send_record(tds, bcpinfo, get_col_data, null_error, offset);
 
-		tds_put_byte(tds, TDS_ROW_TOKEN);   /* 0xd1 */
-		for (i = 0; i < bcpinfo->bindinfo->num_cols; i++) {
-	
-			TDS_INT save_size;
-			unsigned char *save_data;
-			TDSBLOB blob;
-
-			bindcol = bcpinfo->bindinfo->columns[i];
-
-			/*
-			 * Don't send the (meta)data for timestamp columns or
-			 * identity columns unless indentity_insert is enabled.
-			 */
-
-			if ((!bcpinfo->identity_insert_on && bindcol->column_identity) || 
-				bindcol->column_timestamp ||
-				bindcol->column_computed) {
-				continue;
-			}
-
-			rc = get_col_data(bcpinfo, bindcol, offset);
-			if (TDS_FAILED(rc)) {
-				tdsdump_log(TDS_DBG_INFO1, "get_col_data (column %d) failed\n", i + 1);
-				goto cleanup;
-			}
-			tdsdump_log(TDS_DBG_INFO1, "gotten column %d length %d null %d\n",
-					i + 1, bindcol->bcp_column_data->datalen, bindcol->bcp_column_data->is_null);
-
-			save_size = bindcol->column_cur_size;
-			save_data = bindcol->column_data;
-			assert(bindcol->column_data == NULL);
-			if (bindcol->bcp_column_data->is_null) {
-				bindcol->column_cur_size = -1;
-			} else if (is_blob_col(bindcol)) {
-				bindcol->column_cur_size = bindcol->bcp_column_data->datalen;
-				memset(&blob, 0, sizeof(blob));
-				blob.textvalue = (TDS_CHAR *) bindcol->bcp_column_data->data;
-				bindcol->column_data = (unsigned char *) &blob;
-			} else {
-				bindcol->column_cur_size = bindcol->bcp_column_data->datalen;
-				bindcol->column_data = bindcol->bcp_column_data->data;
-			}
-			rc = bindcol->funcs->put_data(tds, bindcol, 1);
-			bindcol->column_cur_size = save_size;
-			bindcol->column_data = save_data;
-
-			if (TDS_FAILED(rc))
-				goto cleanup;
-		}
-	}  /* IS_TDS7_PLUS */
-	else {
-		int row_pos;
-		int row_sz_pos;
-		int blob_cols = 0;
-		int var_cols_written = 0;
-		TDS_INT	 old_record_size = bcpinfo->bindinfo->row_size;
-		unsigned char *record = bcpinfo->bindinfo->current_row;
-
-		memset(record, '\0', old_record_size);	/* zero the rowbuffer */
-
-		/*
-		 * offset 0 = number of var columns
-		 * offset 1 = row number.  zeroed (datasever assigns)
-		 */
-		row_pos = 2;
-
-		rc = TDS_FAIL;
-		if ((row_pos = tds_bcp_add_fixed_columns(bcpinfo, get_col_data, NULL, offset, record, row_pos)) < 0)
-			goto cleanup;
-
-		row_sz_pos = row_pos;
-
-		/* potential variable columns to write */
-
-		if ((row_pos = tds_bcp_add_variable_columns(bcpinfo, get_col_data, NULL, offset, record, row_pos, &var_cols_written)) < 0)
-			goto cleanup;
-
-
-		if (var_cols_written) {
-			TDS_PUT_UA2(&record[row_sz_pos], row_pos);
-			record[0] = var_cols_written;
-		}
-
-		tdsdump_log(TDS_DBG_INFO1, "old_record_size = %d new size = %d \n", old_record_size, row_pos);
-
-		tds_put_smallint(tds, row_pos);
-		tds_put_n(tds, record, row_pos);
-
-		/* row is done, now handle any text/image data */
-
-		blob_cols = 0;
-
-		for (i = 0; i < bcpinfo->bindinfo->num_cols; i++) {
-			bindcol = bcpinfo->bindinfo->columns[i];
-			if (is_blob_type(bindcol->column_type)) {
-				rc = get_col_data(bcpinfo, bindcol, offset);
-				if (TDS_FAILED(rc))
-					goto cleanup;
-				/* unknown but zero */
-				tds_put_smallint(tds, 0);
-				tds_put_byte(tds, bindcol->column_type);
-				tds_put_byte(tds, 0xff - blob_cols);
-				/*
-				 * offset of txptr we stashed during variable
-				 * column processing 
-				 */
-				tds_put_smallint(tds, bindcol->column_textpos);
-				tds_put_int(tds, bindcol->bcp_column_data->datalen);
-				tds_put_n(tds, bindcol->bcp_column_data->data, bindcol->bcp_column_data->datalen);
-				blob_cols++;
-
-			}
-		}
-	}
-
-	tds_set_state(tds, TDS_SENDING);
-	return TDS_SUCCESS;
-
-cleanup:
 	tds_set_state(tds, TDS_SENDING);
 	return rc;
 }
 
+static inline void
+tds5_swap_data(const TDSCOLUMN *col, void *p)
+{
+#ifdef WORDS_BIGENDIAN
+	tds_swap_datatype(tds_get_conversion_type(col->column_type, col->column_size), p);
+#endif
+}
+
 /**
  * Add fixed size columns to the row
  * \param bcpinfo BCP information
@@ -481,8 +504,8 @@ cleanup:
  * \returns new row length or -1 on error.
  */
 static int
-tds_bcp_add_fixed_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data, tds_bcp_null_error ignored, 
-				int offset, unsigned char * rowbuffer, int start)
+tds5_bcp_add_fixed_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data, tds_bcp_null_error null_error,
+			   int offset, unsigned char * rowbuffer, int start)
 {
 	TDS_NUMERIC *num;
 	int row_pos = start;
@@ -493,7 +516,8 @@ tds_bcp_add_fixed_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data
 	assert(bcpinfo);
 	assert(rowbuffer);
 
-	tdsdump_log(TDS_DBG_FUNC, "tds_bcp_add_fixed_columns(%p, %p, ignored, %d, %p, %d)\n", bcpinfo, get_col_data, offset, rowbuffer, start);
+	tdsdump_log(TDS_DBG_FUNC, "tds5_bcp_add_fixed_columns(%p, %p, %p, %d, %p, %d)\n",
+		    bcpinfo, get_col_data, null_error, offset, rowbuffer, start);
 
 	for (i = 0; i < bcpinfo->bindinfo->num_cols; i++) {
 
@@ -503,21 +527,21 @@ tds_bcp_add_fixed_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data
 		if (is_nullable_type(bcpcol->column_type) || bcpcol->column_nullable)
 			continue;
 
-		tdsdump_log(TDS_DBG_FUNC, "tds_bcp_add_fixed_columns column %d is a fixed column\n", i + 1);
+		tdsdump_log(TDS_DBG_FUNC, "tds5_bcp_add_fixed_columns column %d is a fixed column\n", i + 1);
 
 		if (TDS_FAILED(get_col_data(bcpinfo, bcpcol, offset))) {
 			tdsdump_log(TDS_DBG_INFO1, "get_col_data (column %d) failed\n", i + 1);
 			return -1;
 		}
 
-#if USING_SYBEBCNN
-		Let the server reject if no default is defined for the column.
+		/* We have no way to send a NULL at this point, return error to client */
 		if (bcpcol->bcp_column_data->is_null) {
+			tdsdump_log(TDS_DBG_ERROR, "tds5_bcp_add_fixed_columns column %d is a null column\n", i + 1);
 			/* No value or default value available and NULL not allowed. */
-			null_error(bcpinfo, i, offset);
+			if (null_error)
+				null_error(bcpinfo, i, offset);
 			return -1;
 		}
-#endif
 
 		if (is_numeric_type(bcpcol->column_type)) {
 			num = (TDS_NUMERIC *) bcpcol->bcp_column_data->data;
@@ -538,6 +562,7 @@ tds_bcp_add_fixed_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data
 			cpbytes = bcpcol->bcp_column_data->datalen > column_size ?
 				  column_size : bcpcol->bcp_column_data->datalen;
 			memcpy(&rowbuffer[row_pos], bcpcol->bcp_column_data->data, cpbytes);
+			tds5_swap_data(bcpcol, &rowbuffer[row_pos]);
 
 			/* CHAR data may need padding out to the database length with blanks */
 			/* TODO check binary !!! */
@@ -566,7 +591,8 @@ tds_bcp_add_fixed_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data
  * \return length of (potentially modified) rowbuffer, or -1.
  */
 static int
-tds_bcp_add_variable_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data, tds_bcp_null_error null_error, int offset, TDS_UCHAR* rowbuffer, int start, int *pncols)
+tds5_bcp_add_variable_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_data, tds_bcp_null_error null_error,
+			      int offset, TDS_UCHAR* rowbuffer, int start, int *pncols)
 {
 	TDS_USMALLINT offsets[256];
 	unsigned int i, row_pos;
@@ -612,15 +638,14 @@ tds_bcp_add_variable_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_d
 		if (TDS_FAILED(get_col_data(bcpinfo, bcpcol, offset)))
 			return -1;
 
-#if USING_SYBEBCNN
-		/* If it's a NOT NULL column, and we have no data, throw an error. */
-		No, the column could have a default defined.
-		if (!(bcpcol->column_nullable) && bcpcol->bcp_column_data->is_null) {
+		/* If it's a NOT NULL column, and we have no data, throw an error.
+		 * This is the behavior for Sybase, this function is only used for Sybase */
+		if (!bcpcol->column_nullable && bcpcol->bcp_column_data->is_null) {
 			/* No value or default value available and NULL not allowed. */
-			null_error(bcpinfo, i, offset);
+			if (null_error)
+				null_error(bcpinfo, i, offset);
 			return -1;
 		}
-#endif
 
 		/* move the column buffer into the rowbuffer */
 		if (!bcpcol->bcp_column_data->is_null) {
@@ -635,6 +660,7 @@ tds_bcp_add_variable_columns(TDSBCPINFO *bcpinfo, tds_bcp_get_col_data get_col_d
 				cpbytes = bcpcol->bcp_column_data->datalen > bcpcol->column_size ?
 				bcpcol->column_size : bcpcol->bcp_column_data->datalen;
 				memcpy(&rowbuffer[row_pos], bcpcol->bcp_column_data->data, cpbytes);
+				tds5_swap_data(bcpcol, &rowbuffer[row_pos]);
 			}
 		}
 
@@ -843,6 +869,9 @@ tds_bcp_start(TDSSOCKET *tds, TDSBCPINFO *bcpinfo)
 
 	tdsdump_log(TDS_DBG_FUNC, "tds_bcp_start(%p, %p)\n", tds, bcpinfo);
 
+	if (!IS_TDS50_PLUS(tds->conn))
+		return TDS_FAIL;
+
 	rc = tds_submit_query(tds, bcpinfo->insert_stmt);
 	if (TDS_FAILED(rc))
 		return rc;
