@@ -74,7 +74,7 @@
 #include <freetds/configs.h>
 #include <freetds/utils/string.h>
 #include <freetds/utils.h>
-#include "replacements.h"
+#include <freetds/replacements.h>
 
 static int tds_config_login(TDSLOGIN * connection, TDSLOGIN * login);
 static int tds_config_env_tdsdump(TDSLOGIN * login);
@@ -91,15 +91,15 @@ static char *interf_file = NULL;
 
 #define TDS_ISSPACE(c) isspace((unsigned char ) (c))
 
+const char STD_DATETIME_FMT[] = "%b %e %Y %I:%M%p";
+
 #if !defined(_WIN32) && !defined(DOS32X)
-       const char STD_DATETIME_FMT[] = "%b %e %Y %I:%M%p";
 static const char pid_config_logpath[] = "/tmp/tdsconfig.log.%d";
 static const char freetds_conf[] = "%s/etc/freetds.conf";
 static const char location[] = "(from $FREETDS/etc)";
 static const char pid_logpath[] = "/tmp/freetds.log.%d";
 static const char interfaces_path[] = "/etc/freetds";
 #else
-       const char STD_DATETIME_FMT[] = "%b %d %Y %I:%M%p"; /* msvcr80.dll does not support %e */
 static const char pid_config_logpath[] = "c:\\tdsconfig.log.%d";
 static const char freetds_conf[] = "%s\\freetds.conf";
 static const char location[] = "(from $FREETDS)";
@@ -886,16 +886,17 @@ tds_config_env_tdshost(TDSLOGIN * login)
 #define TDS_FIND(k,b,c) tds_find(k, b, TDS_VECTOR_SIZE(b), sizeof(b[0]), c)
 
 
-static void *
+static const void *
 tds_find(const void *key, const void *base, size_t nelem, size_t width,
          int (*compar)(const void *, const void *))
 {
-	size_t i;
-	for (i=0; i < nelem; i++) {
-		char *p = (char*)base + width * i;
-		if (0 == compar(key, p)) {
+	size_t n;
+	const char *p = (const char*) base;
+
+	for (n = nelem; n != 0; --n) {
+		if (0 == compar(key, p))
 			return p;
-		}
+		p += width;
 	}
 	return NULL;
 }
@@ -907,10 +908,10 @@ struct tdsvername_t
 };
 
 static int
-tds_vernanme_cmp(const void *key, const void *pelem)
+tds_vername_cmp(const void *key, const void *pelem)
 {
 	return strcmp((const char *)key, ((const struct tdsvername_t *)pelem)->name);
-}	
+}
 
 /**
  * Set TDS version from given string
@@ -925,15 +926,10 @@ tds_config_verstr(const char *tdsver, TDSLOGIN * login)
 		{ {   "0", 0x000 }
 		, {"auto", 0x000 }
 		, { "4.2", 0x402 }
-		, { "4.2", 0x402 }
-		, {  "46", 0x406 }
-		, { "4.6", 0x406 }
 		, {  "50", 0x500 }
 		, { "5.0", 0x500 }
 		, {  "70", 0x700 }
 		, { "7.0", 0x700 }
-		, {  "80", 0x701 }
-		, { "8.0", 0x701 }
 		, { "7.1", 0x701 }
 		, { "7.2", 0x702 }
 		, { "7.3", 0x703 }
@@ -946,7 +942,7 @@ tds_config_verstr(const char *tdsver, TDSLOGIN * login)
 		return NULL;
 	}
 
-	if ((pver = (const struct tdsvername_t *) TDS_FIND(tdsver, tds_versions, tds_vernanme_cmp)) == NULL) {
+	if ((pver = (const struct tdsvername_t *) TDS_FIND(tdsver, tds_versions, tds_vername_cmp)) == NULL) {
 		tdsdump_log(TDS_DBG_INFO1, "error: no such version: %s\n", tdsver);
 		return NULL;
 	}
