@@ -47,7 +47,7 @@
 #include <freetds/tds.h>
 #include <freetds/convert.h>
 #include <freetds/bytes.h>
-#include "replacements.h"
+#include <freetds/replacements.h>
 
 typedef unsigned short utf16_t;
 
@@ -2955,6 +2955,17 @@ tds_get_null_type(TDS_SERVER_TYPE srctype)
 	return srctype;
 }
 
+static void
+two_digit(char *out, int num)
+{
+	if (num < 1)
+		num = 1;
+	if (num > 31)
+		num = 31;
+	out[0] = num < 10 ? ' ' : num/10 + '0';
+	out[1] = num%10 + '0';
+}
+
 /**
  * format a date string according to an "extended" strftime(3) formatting definition.
  * @param buf     output buffer
@@ -2972,6 +2983,7 @@ tds_strftime(char *buf, size_t maxsize, const char *format, const TDSDATEREC * d
 	size_t length;
 	char *our_format;
 	char *pz = NULL;
+	bool z_found = false;
 	
 	assert(buf);
 	assert(format);
@@ -3002,27 +3014,52 @@ tds_strftime(char *buf, size_t maxsize, const char *format, const TDSDATEREC * d
 
 	strcpy(our_format, format);
 
-	/*
-	 * Look for "%z" in the format string.  If found, replace it with dr->milliseconds.
-	 * For example, if milliseconds is 124, the format string
-	 * "%b %d %Y %H:%M:%S.%z" would become
-	 * "%b %d %Y %H:%M:%S.124".
-	 */
-	for (pz = our_format; (pz = strstr(pz, "%z")) != NULL; pz++) {
-		/* Skip any escaped cases (%%z) */
-		if (pz > our_format && *(pz - 1) != '%')
-			break;
-	}
+	for (pz = our_format; *pz; ) {
+		if (*pz++ != '%')
+			continue;
 
-	if (pz) {
-		if (prec || pz <= our_format || pz[-1] != '.') {
-			char buf[12];
-			sprintf(buf, "%07d", dr->decimicrosecond);
-			memcpy(pz, buf, prec);
-			strcpy(pz + prec, format + (pz - our_format) + 2);
-		} else {
-			strcpy(pz - 1, format + (pz - our_format) + 2);
+		switch (*pz) {
+		case 0:
+			continue;
+		case '%':
+			/* escaped, do not treat as format on next iteration */
+			break;
+		case 'e':
+			/* not portable: day of month, single digit preceded by a blank */
+			/* not supported on old Windows versions */
+			two_digit(pz-1, dr->day);
+			break;
+		case 'l':
+			/* not portable: 12-hour, single digit preceded by a blank */
+			/* not supported on: SCO Unix, AIX, HP-UX, Windows
+			 * supported on: *BSD, MacOS, Linux, Solaris */
+			two_digit(pz-1, (dr->hour + 11u) % 12u + 1);
+			break;
+		case 'z':
+			/*
+			 * Look for "%z" in the format string.  If found, replace it with dr->milliseconds.
+			 * For example, if milliseconds is 124, the format string
+			 * "%b %d %Y %H:%M:%S.%z" would become
+			 * "%b %d %Y %H:%M:%S.124".
+			 */
+			if (z_found)
+				break;
+			z_found = true;
+
+			--pz;
+			if (prec || pz <= our_format || pz[-1] != '.') {
+				char buf[12];
+				sprintf(buf, "%07d", dr->decimicrosecond);
+				memcpy(pz, buf, prec);
+				strcpy(pz + prec, format + (pz - our_format) + 2);
+				pz += prec;
+			} else {
+				strcpy(pz - 1, format + (pz - our_format) + 2);
+				pz--;
+			}
+			continue;
 		}
+		++pz;
 	}
 
 	length = strftime(buf, maxsize, our_format, &tm);
