@@ -33,7 +33,7 @@
 #include <freetds/tls.h>
 #include <freetds/checks.h>
 #include <freetds/utils/string.h>
-#include "replacements.h"
+#include <freetds/replacements.h>
 #include <freetds/enum_cap.h>
 #include <freetds/utils.h>
 
@@ -332,7 +332,7 @@ tds_free_param_result(TDSPARAMINFO * param_info)
 		TDS_ZERO_FREE(param_info->columns);
 
 	/*
-	 * NOTE some information should be freed too but when this function 
+	 * NOTE some informations should be freed too but when this function 
 	 * is called are not used. I hope to remove the need for this
 	 * function ASAP
 	 * A better way is to support different way to allocate and get
