@@ -45,7 +45,7 @@
 #include <freetds/thread.h>
 #include <freetds/utils/string.h>
 #include <freetds/bool.h>
-#include "replacements.h"
+#include <freetds/replacements.h>
 
 /**
  * \ingroup libtds
@@ -213,7 +213,7 @@ tds_sspi_get_auth(TDSSOCKET * tds)
 	if (!tds_init_secdll())
 		return NULL;
 
-	/* parse username/password information */
+	/* parse username/password informations */
 	memset(&identity, 0, sizeof(identity));
 	user_name = tds_dstr_cstr(&login->user_name);
 	if ((p = strchr(user_name, '\\')) != NULL) {
